# main.py
# Name: Alex Cappiello
# AndrewID: acappiel
# Section: H
# Term project for 15-110, spring 2011
# All code original work unless otherwise noted

from Tkinter import *
try:
    from PIL import Image, ImageTk, ImageDraw
    pil = True
except ImportError:
    raise #PIL not installed. Cannot run.
import Gui, FileIO, DrawingTools, MouseOps

################################################################################

def drawImage():
    #make an image that Tkinter can handle and display it
    photo = ImageTk.PhotoImage( canvas.data.rawImage )
    canvas.create_image( canvas.data.drawOffset, canvas.data.drawOffset,
                         anchor = NW,image = photo)
    canvas.data.tkImage = photo

################################################################################

def leftClick( event ):
    x = canvas.canvasx( event.x )
    y = canvas.canvasy( event.y )
    #remember the initial position
    canvas.data.x = x
    canvas.data.y = y
    #call the event for the selected tool
    canvas.data.action.leftClick( x, y )
    canvas.data.unsavedChanges = True

def rightClick( event ):
    x = canvas.canvasx( event.x )
    y = canvas.canvasy( event.y )
    #call the event for the selected tool
    canvas.data.action.rightClick( x, y )
    canvas.data.unsavedChanges = True

def mouseDrag( event ):
    x = canvas.canvasx( event.x )
    y = canvas.canvasy( event.y )
    #call the event for the selected tool
    canvas.data.action.drag( x, y )

def mouseRelease( event ):
    x = canvas.canvasx( event.x )
    y = canvas.canvasy( event.y )
    #call the event for the selected tool
    canvas.data.action.release( x, y )
    redrawAll()

def keyPressed( event ):
    #redrawAll()
    pass

def timerFired():
    if ( canvas.data.redraw ):
        redrawAll()
        canvas.data.redraw = False
    delay = 15 # milliseconds
    canvas.after( delay, timerFired ) # pause, then call timerFired again

################################################################################

def redrawAll():
    canvas.delete( ALL )
    drawImage()

################################################################################

def init( root ):
    root.title( "Image Editor" )
    try:
        root.state( "zoomed" ) #maximize on Windows
    except:
        pass
    #some other modules need to access the canvas and root
    DrawingTools.canvas = canvas
    MouseOps.canvas = canvas
    FileIO.canvas = canvas
    FileIO.root = root
    #can't in Gui because configure() called before canvas created
    canvas.data.colorButtonA = Gui.colorA
    canvas.data.colorButtonB = Gui.colorB
    #reasonable starting image dimensions
    initialWidth = 800; initialHeight = 600
    DrawingTools.blankImage( initialWidth, initialHeight )
    #sets some defaults
    canvas.data.rgbColorA = ( 0, 0, 0 )
    canvas.data.hexColorA = "#000000"
    canvas.data.rgbColorB = ( 255, 255, 255 )
    canvas.data.hexColorB = "#ffffff"
    canvas.data.activeToolName = "Brush"
    canvas.data.unsavedChanges = False
    #create pointers to image
    DrawingTools.configure()

################################################################################

def run():
    # create the root
    root = Tk()
    #make menus and toolbars first
    Gui.configure( root )
    global canvas
    #somewhat arbitrary dimensions, as I maximaize the window in init()
    #border makes it easy to see edge of drawing area
    width = 800; height = 600; border = 5; offset = 3 + border
    canvas = Canvas( root, width = width, height = height, bg = "#ffffff",
                     relief = RIDGE, borderwidth = border,
                     xscrollcommand = Gui.scrollX.set,
                     yscrollcommand = Gui.scrollY.set,
                     scrollregion = ( offset, offset, width, height ) )
    Gui.scrollX.config( command = canvas.xview )
    Gui.scrollY.config( command = canvas.yview )
    canvas.pack()
    # Store canvas in root and in canvas itself for callbacks
    root.canvas = canvas.canvas = canvas
    class Struct: pass
    canvas.data = Struct()
    canvas.data.pil = pil
    #offset positions image properly to account for Tkinter acting odd
    canvas.data.drawOffset = offset
    init( root )
    # set up events
    canvas.bind( "<Button-1>", leftClick )
    canvas.bind( "<Button-3>", rightClick )
    canvas.bind( "<B1-Motion>", mouseDrag )
    canvas.bind( "<ButtonRelease-1>", mouseRelease )
    root.bind( "<Key>", keyPressed )
    timerFired()
    # and launch the app
    root.mainloop()  # This call BLOCKS

run()
